mMinimap = LibStub("AceAddon-3.0"):NewAddon("mMinimap", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0")
local mMinimap = mMinimap


-- Locals
local db

local TimeOnEnter, TimeOnLeave
local TrackOnEnter, TrackOnLeave
local BackdropFframe
local PostionXWidget, PostionYWidget
local mMinimapIcon = [[Interface\Icons\INV_Offhand_Sunwell_D_01]]

local CoordsFormat

local backdrop = {
	insets = {
	},
}

local LSM = LibStub("LibSharedMedia-3.0")

CoordsFormat = ("%%.%df,%%.%df"):format(1, 1)

function mMinimap:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			MoveLFDFrame = true,
			HideTracking = true,
			MinimapLocked = true,
			MinimapScale = 1.5,
			MinimapPositionX = 0,
			MinimapPositionY = 0,
			MinimapAnchor = "UIParent",
			MinimapShape = "SQUARE",
			
			-- Clock
			HideClock = false,
			
			-- Coords
			ShowCoords = false,
			CoordsPrecition = 0,
			
			-- Mail
			ReplaceMailIcon = true,
			
			-- Font
			ZoneFont = "accid",
			ZoneFontSize = 14,
			ZoneFontFlags = "OUTLINE",
			ZoneFontColor = {1, 1, 1, 1},
			ZoneTextX = 0,
			ZoneTextY = 3,
			ZoneFontShadow = 0,
			
			-- Backdrop
			ShowBackdrop = true,
			BackdropExtent = 4,
			BGInset = 2,
			BGBorderTexture = "None",
			BGTexture = "Solid",
			BGColor = {22/255, 31/255, 30/255, 1},
			BGBorderColor = {1, 1, 1, 1},
			BGBorderSize = 10,
			
			-- Frame movers
			ShowFrameAnchors = false,
			FramePositions = {
			},
			WatchFrameHeight = (WatchFrame:GetHeight() / (WatchFrame:GetEffectiveScale() * UIParent:GetEffectiveScale())),
			WatchFrameWidth = (WatchFrame:GetWidth() / (WatchFrame:GetEffectiveScale()  * UIParent:GetEffectiveScale())),
		}
	}
	
	self.db  = LibStub("AceDB-3.0"):New("mMinimapDB", defaults, "Default")
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	self:SetupLDB()
	
	self:SetEnabledState(db.Enabled)
	
	self:RegisterEvent("VARIABLES_LOADED")
end

function mMinimap:OnEnable()
	
	-- Register Events
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "NewArea")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "NewArea")
	self:RegisterEvent("UPDATE_INSTANCE_INFO", "NewArea")
	self:RegisterEvent("CALENDAR_UPDATE_PENDING_INVITES")
	
	-- Register slash command
	self:RegisterChatCommand("mm", "ToggleConfig")
	
	mMinimap:OneTimeChanges()
	mMinimap:Update()
	mMinimap:ShowHideFrames()		
	self:NewArea()
	
	MinimapCluster:SetMovable(true)
	MinimapCluster:StartMoving()
  	MinimapCluster:StopMovingOrSizing()
	
	local UIFSettings = {
		s = {
			font = LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"),
			fontSize = 16,			
		}
	}
	local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mMinimap", UIFSettings)
	
	self:SecureHook(DurabilityFrame, "SetPoint", "HookedSetPoint")
	self:SecureHook(VehicleSeatIndicator, "SetPoint", "HookedSetPoint")
	self:SecureHook(WatchFrame, "SetPoint", "HookedSetPoint")
	self:SecureHook(TicketStatusFrame, "SetPoint", "HookedSetPoint")
	self:SecureHook(WorldStateAlwaysUpFrame, "SetPoint", "HookedSetPoint")
	self:SecureHook("WorldStateAlwaysUpFrame_Update")
	self:WorldStateAlwaysUpFrame_Update()
end

function mMinimap:OnDisable()
end

function mMinimap:ProfileChanged(...)
	db = self.db.profile
	self:ChangeCallback()
	self:SetAllFramePositions()
end


function mMinimap:ShowHideFrames()
	-- Movable Minimap
	if not db.MinimapLocked then
		Minimap:RegisterForDrag("LeftButton")
		MinimapCluster:EnableMouse(true)			
	else
		Minimap:RegisterForDrag(nil)
		MinimapCluster:EnableMouse(false)
	end	
	
	-- Clock
	if db.HideClock then
		TimeManagerClockButton:Hide()
		TimeManagerClockTicker:Hide()
	else
		TimeManagerClockButton:Show()
		TimeManagerClockTicker:Show()
	end
	
	-- Backdrop
	if db.ShowBackdrop then
		BackdropFframe:Show()
	else
		BackdropFframe:Hide()
	end
	
	-- Coordinates
	if db.ShowCoords then		
		MinimapCoordinates:Show()
	else
		MinimapCoordinates:Hide()
	end
	
	-- Mail
	if db.ReplaceMailIcon then
		MiniMapMailIcon:Hide()
		MiniMapMailFrame.Text:Show()
	else
		MiniMapMailIcon:Show()
		if MiniMapMailFrame.Text then
			MiniMapMailFrame.Text:Hide()
		end
	end	
	
	MinimapCluster:ClearAllPoints()
	local scale = db.MinimapScale
	if not db.MinimapPositionSet then
		MinimapCluster:SetPoint("TOPRIGHT", -10, -30)
	else
		MinimapCluster:SetPoint("CENTER", _G[db.MinimapAnchor], "CENTER", db.MinimapPositionX / scale, db.MinimapPositionY / scale)
	end
	MinimapCluster:SetScale(scale)
	
	-- Tracking
	if db.HideTracking then
		MiniMapTracking:Hide()
	else
		MiniMapTracking:Show()		
	end	
	
	if db.ShowFrameAnchors then
		self:ShowFrameAnchors()
	else
		self:HideFrameAnchors()
	end
	
	self:SetAllFramePositions()
end

function mMinimap:Update()
	if BackdropFframe and db.ShowBackdrop then
		BackdropFframe:Show()
	elseif BackdropFframe then
		BackdropFframe:Hide()
	end
		
	-- Zone Text
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneTextButton:SetPoint("BOTTOM", Minimap, "TOP", db.ZoneTextX, db.ZoneTextY)
	MinimapZoneTextButton:SetWidth(Minimap:GetWidth() * 1.5)
	MinimapZoneText:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)	
	MinimapZoneText:SetShadowOffset(db.ZoneFontShadow, -db.ZoneFontShadow)	
	
	-- Clock		
	TimeManagerClockTicker:ClearAllPoints()	
	TimeManagerClockTicker:SetPoint("BOTTOMLEFT", 3, 2)
	TimeManagerClockTicker:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)
	TimeManagerClockTicker:SetTextColor(unpack(db.ZoneFontColor))	
	TimeManagerClockTicker:SetShadowOffset(db.ZoneFontShadow, -db.ZoneFontShadow)
	
	-- Looking for Dungeon (Tooltip)
	if db.MoveLFDFrame then
		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("CENTER", UIParent)
	else
		LFDSearchStatus:ClearAllPoints()
		LFDSearchStatus:SetPoint("RIGHT", MiniMapLFGFrame, "LEFT")
	end	
	
	-- Minimap Shape
	if db.MinimapShape == "SQUARE" then
		Minimap:SetMaskTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	else
		Minimap:SetMaskTexture([=[Textures\MinimapMask]=])
	end
	
	-- Coords format
	CoordsFormat = ("%%.%df,%%.%df"):format(db.CoordsPrecition, db.CoordsPrecition)
	
	-- Backdrop
	backdrop.insets.left = db.BGInset
	backdrop.insets.right = db.BGInset
	backdrop.insets.top = db.BGInset
	backdrop.insets.bottom = db.BGInset
	backdrop.bgFile = LSM:Fetch("background", db.BGTexture)
	backdrop.edgeFile = LSM:Fetch("border", db.BGBorderTexture)
	backdrop.edgeSize = db.BGBorderSize
	BackdropFframe:SetBackdrop(backdrop)
	BackdropFframe:SetBackdropColor(unpack(db.BGColor))
	BackdropFframe:SetBackdropBorderColor(unpack(db.BGBorderColor))
	
	BackdropFframe:SetPoint("TOPLEFT", -db.BackdropExtent, db.BackdropExtent)	
	BackdropFframe:SetPoint("BOTTOMRIGHT", db.BackdropExtent, -db.BackdropExtent)
	
	-- Coordinates	
	MinimapCoordinates:SetPoint("BOTTOM", 0, 2)
	MinimapCoordinates:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize, db.ZoneFontFlags)
	MinimapCoordinates:SetTextColor(unpack(db.ZoneFontColor))
	MinimapCoordinates:SetShadowOffset(db.ZoneFontShadow, -db.ZoneFontShadow)
	
	-- Mail
	MiniMapMailFrame.Text:SetFont(LSM:Fetch("font", db.ZoneFont), db.ZoneFontSize > 2 and db.ZoneFontSize - 2 or db.ZoneFontSize, db.ZoneFontFlags)
	MiniMapMailFrame.Text:SetTextColor(unpack(db.ZoneFontColor))
	
end

function mMinimap:OneTimeChanges()
	Minimap:SetClampedToScreen(true)
	Minimap:SetScript('OnDragStop', function() 
		MinimapCluster:StopMovingOrSizing() 
		local scale = MinimapCluster:GetEffectiveScale() / UIParent:GetEffectiveScale()
		
		x, y = MinimapCluster:GetCenter()
		x, y = x * scale, y * scale
		
		x = x - GetScreenWidth() / 2
		y = y - GetScreenHeight() / 2
		
		db.MinimapPositionX, db.MinimapPositionY = x, y
		
		if PostionXWidget then
			PostionXWidget:SetValue(db.MinimapPositionX)
			PostionYWidget:SetValue(db.MinimapPositionY)
		end
		
		db.MinimapPositionSet = true
	end)
	Minimap:SetScript('OnDragStart', function() 
		MinimapCluster:StartMoving() 
	end)
	
	-- Mousewheel zoom
	MinimapZoomIn:Hide()
	MinimapZoomOut:Hide()
	Minimap:EnableMouseWheel()
	Minimap:SetScript('OnMouseWheel', function(self, direction)
		if(direction > 0) then
			MinimapZoomIn:Click()
		else
			MinimapZoomOut:Click()
		end
	end)
	
	-- Zone text
	MinimapZoneTextButton:SetParent(Minimap)
	MinimapZoneText:SetAllPoints(MinimapZoneTextButton)
	
	-- Tracking
	MiniMapTracking:ClearAllPoints()
	MiniMapTracking:SetPoint("TOPLEFT", Minimap, "TOPLEFT")	
	MiniMapTrackingButtonBorder:Hide()
	MiniMapTrackingIcon:SetTexCoord(0.065, 0.935, 0.065, 0.935)	
	MiniMapTrackingIconOverlay:SetTexture(nil)
	MiniMapTrackingButton:SetHighlightTexture(nil)
	self:HookScript(MiniMapTrackingButton, "OnEnter", function(self)
		Minimap:GetScript("OnEnter")(Minimap)
	end)
	self:HookScript(MiniMapTrackingButton, "OnLeave", function(self)
		Minimap:GetScript("OnLeave")(Minimap)
	end)
	
	-- Time/Calendar
	GameTimeFrame:Hide()
	
	-- New Mail
	MiniMapMailFrame:ClearAllPoints()
	MiniMapMailFrame:SetPoint("TOP")	
	MiniMapMailBorder:SetTexture(nil)
	if not MiniMapMailFrame.Text then
		MiniMapMailFrame.Text = MiniMapMailFrame:CreateFontString(nil, nil, "GameFontNormal" )		
		MiniMapMailFrame.Text:SetText("New Mail!")
	end

	-- World map button
	MiniMapWorldMapButton:Hide()
	
	-- Looking for Dungeon (Icon)
	MiniMapLFGFrame:ClearAllPoints()
	MiniMapLFGFrame:SetPoint("TOP", MiniMapTracking, "BOTTOM")
	
	-- Battleground
	MiniMapBattlefieldFrame:ClearAllPoints()
	MiniMapBattlefieldFrame:SetPoint("TOPRIGHT", Minimap)
	MiniMapBattlefieldBorder:SetTexture(nil)

	-- Instance Difficulty
	MiniMapInstanceDifficulty:ClearAllPoints()
	MiniMapInstanceDifficulty:SetPoint("BOTTOMRIGHT", Minimap)
	
	-- Backdrop
	if not BackdropFframe then
		BackdropFframe = CreateFrame("Frame", "mMinimapBackdropFrame", Minimap)			
		BackdropFframe:SetFrameLevel(1)
	end
	
	-- Coordinates
	if not MinimapCoordinates then
		MinimapCoordinates = Minimap:CreateFontString(nil, nil, "GameFontNormal")		
	end	
	
	-- Clock
	TimeManagerClockButton:GetRegions():Hide()
	
	-- LFG
	MiniMapLFGFrameBorder:Hide()	
	
	-- Border
	MinimapBorder:SetTexture(nil)
	MinimapBorderTop:Hide()
	
	-- Compas
	MinimapNorthTag:SetAlpha(0)	
end

---------------------------------------------------
---------------------------------------------------
-- Events
---------------------------------------------------
Minimap:SetScript("OnMouseUp", function(self, button)
	if (button == 'RightButton') and  db.HideTracking then
		ToggleDropDownMenu(1, nil, MiniMapTrackingDropDown, self, 0, 0)
	elseif (button == "MiddleButton") then
		ToggleCalendar()
	else
		Minimap_OnClick(self, button)
	end
end)

Minimap:SetScript("OnEnter", function()
	MinimapCoordinates:Show()
	TimeManagerClockTicker:Show()
	MiniMapTracking:Show()
end)

Minimap:SetScript("OnLeave", function()
	mMinimap:ShowHideFrames()
end)

function mMinimap:NewArea(event)
	SetMapToCurrentZone()
end

function mMinimap:CALENDAR_UPDATE_PENDING_INVITES()
	if(CalendarGetNumPendingInvites() ~= 0) then
		TimeManagerClockTicker:SetTextColor(0, 1, 0)
	else
		TimeManagerClockTicker:SetTextColor(1, 1, 1)
	end
end

function mMinimap:VARIABLES_LOADED()
	SetCVar('showClock', '1')

	LoadAddOn("Blizzard_TimeManager")	
	
	self:HookScript(TimeManagerClockButton, "OnEnter", function(self)
		Minimap:GetScript("OnEnter")(Minimap)
	end)
	
	self:HookScript(TimeManagerClockButton, "OnLeave", function(self)
		Minimap:GetScript("OnLeave")(Minimap)
	end)
	
	TimeManagerClockTicker:SetParent(Minimap)
	TimeManagerClockTicker:ClearAllPoints()
	TimeManagerClockButton:ClearAllPoints()
	TimeManagerClockButton:SetAllPoints(TimeManagerClockTicker)
	TimeManagerClockButton:Show()
	TimeManagerClockButton:SetScript('OnClick', function(self, button)
		if(button == 'RightButton') then
			ToggleCalendar()
		else
			if(self.alarmFiring) then
				PlaySound('igMainMenuQuit')
				TimeManager_TurnOffAlarm()
			else
				ToggleTimeManager()
			end
		end
	end)
	
	-- Show the clock if an alarm goes of, and in red
	TimeManagerAlarmFiredTexture.Show = function() TimeManagerClockTicker:SetTextColor(1, 0, 0) TimeManagerClockTicker:Show() end
	TimeManagerAlarmFiredTexture.Hide = function() TimeManagerClockTicker:SetTextColor(1, 1, 1) end	
end

---------------------------------------------------
---------------------------------------------------

local UpdateFrame = CreateFrame("Frame", "mMinimapUpdateFrame", UIParent)
UpdateFrame:SetScript("OnUpdate", function(self, elapsed)
	self.elapsed = (self.elapsed or 0) + elapsed
	if self.elapsed > 0.2 then
		if MinimapCoordinates then
			local x, y = GetPlayerMapPosition("player")
			if x and y then
				MinimapCoordinates:SetText(CoordsFormat:format(x * 100, y * 100))
			end
		end
		self.elapsed = 0
	end	
end)

function mMinimap:SetupLDB()
	local LDB = LibStub("LibDataBroker-1.1", true)
	if not LDB then return end
	
	local l = LDB:NewDataObject("mMinimap")
	l.type = "launcher"
	l.icon = mMinimapIcon
	l.tocname = "mMinimap"
	l.label = "mMinimap"
	l.OnClick = function(self, button)
		mMinimap:ToggleConfig()
	end
	l.OnTooltipShow = function(tt)
		tt:AddLine("mMinimap")
		tt:AddLine("Click to toggle configuration")
	end
	
	self.ldbojb = l
end

function GetMinimapShape() return db.MinimapShape end